#!/usr/bin/env python3
import os
import argparse
from Bio import SeqIO
from Bio.Alphabet import generic_dna
from operator import itemgetter
import numpy
import math

#~ def main():
#~ 
    #~ parser = argparse.ArgumentParser(description="This program screens a set of genes in a fasta file.")
    #~ parser.add_argument('-i', nargs='?', type=str, help='List of genes files (list of fasta files)', required=True)
    #~ parser.add_argument('-t', nargs='?', type=float, help='threshold', required=False)
    #~ parser.add_argument('-c', nargs='?', type=bool, help='All alleles must be conserved', required=False)
    #~ parser.add_argument('-r', nargs='?', type=bool, help='Return values', required=False)
#~ 
    #~ args = parser.parse_args()
    #~ genes = args.i
#~ 
#~ 
    #~ try:
        #~ threshold=float(args.t)
    #~ except:
        #~ threshold=0.05
        #~ pass
    #~ try:
        #~ OneNotConserved=bool(args.c)
    #~ except:
        #~ OneNotConserved=False
        #~ pass
#~ 
    #~ try:
        #~ ReturnValues=bool(args.r)
    #~ except:
        #~ ReturnValues=False
        #~ pass
#~ 
    #~ return getStats(genes,threshold,OneNotConserved,ReturnValues)



def main(genes,threshold,OneNotConserved,ReturnValues,logScale,outputpath,split_thresh):

    gene_fp = open( genes, 'r')

    statistics=[]

    genenumber=0
    conservedlengthgenes=[]
    notconservedlengthgenes=[]
    genesWoneAllele=[]
    z=0
    print("Genes with only 1 allele:")
    modaStats=[]
    allsizes=[]
    tabStats=[]
    allAllelesStats=[]
    allNumberAlleles=[]
    allNumberAllelesMean=[]
    allNumberAllelesMedian=[]
    genesList=[]

    htmlgenespath=os.path.join(outputpath,"genes_html/")
    relpath=os.path.relpath(htmlgenespath,outputpath)

    #get stats for each locus file
    for gene in gene_fp:

        gene = gene.rstrip('\n')

        genesList.append(gene)
        #gene_fp2 = HTSeq.FastaReader(gene)
        maxsize=0
        minsize=99999999999
        sizesum=0
        allelenumber=0
        aux=[0,0]
        sizes=[]


        #per gene get all sizes, minimin size, maximum size, media and mode
        for allele in SeqIO.parse(gene, "fasta", generic_dna):

            allelesize=len(str(allele.seq))
            sizes.append(allelesize)
            if 	allelesize>=maxsize or allelesize<=minsize:
                if allelesize>maxsize:
                    maxsize=allelesize
                if allelesize<minsize:
                    minsize=allelesize
            else:
                aux.append(allelesize)
            sizesum+=allelesize
            allelenumber+=1

        allAllelesStats.append(allelenumber)

        aux[0]=	minsize
        #mean=float(sizesum)/allelenumber

        aux[1]=	maxsize

        sizesnpList=numpy.array(sizes)
        tabStats.append([gene,numpy.amin(sizesnpList),numpy.amax(sizesnpList),numpy.mean(sizesnpList),numpy.std(sizesnpList)])

        i=0

        moda=max(set(sizes), key=sizes.count)
        median=numpy.median(numpy.array(sizes))
        mean=numpy.mean(numpy.array(sizes))

        modaStats.append(moda)


        #get ratio between number of alleles outside conserved threshold
        for size in sizes:

            if (not float(size)> moda*(1+threshold) and not float(size)< moda*(1-threshold)):
                i+=1
        rate = i/float(allelenumber)

        #check if the gene is conserved considering the threshold and the -p parameter
        #get locus with only 1 allele
        if not OneNotConserved and (rate>=1 or (len(sizes)-i)<2) :
            if allelenumber==1:
                z+=1
                genesWoneAllele.append(os.path.join(relpath,(os.path.basename(gene)).replace(".fasta",".html")))

                print(os.path.basename(gene))
            conservedlengthgenes.append(os.path.basename(gene))
        elif OneNotConserved and rate>=1  :
            #print i,allelenumber
            if allelenumber==1:
                z+=1
                genesWoneAllele.append(os.path.join(relpath,(os.path.basename(gene)).replace(".fasta",".html")))

                print(os.path.basename(gene))
            conservedlengthgenes.append(os.path.basename(gene))
        else:
            #notconservedlengthgenes.append(os.path.basename(gene))
            notconservedlengthgenes.append(os.path.join(relpath,(os.path.basename(gene)).replace(".fasta",".html")))



        sizes.append(gene)
        sizes.append(sizes[0])
        sizes.append(median)



        allsizes.append(sizes)
        if logScale:
            allNumberAlleles.append([gene,math.log10(moda),math.log10(allelenumber)])
            allNumberAllelesMean.append([gene,math.log10(mean),math.log10(allelenumber)])
            allNumberAllelesMedian.append([gene,math.log10(median),math.log10(allelenumber)])
        else:
            allNumberAlleles.append([gene,moda,allelenumber])
            allNumberAllelesMean.append([gene,mean,allelenumber])
            allNumberAllelesMedian.append([gene,median,allelenumber])

    print("\n"+str(z)+ " genes with only one allele\n")

    #order genes by median
    sortbymedia=sorted(allsizes, key=itemgetter(-1))
    #sortbymedia.reverse()

    #order genes by number of alleles
    sortbyNumberAlleles=sorted(allNumberAlleles, key=itemgetter(1))
    sortbyNumberAllelesMean=sorted(allNumberAllelesMean, key=itemgetter(1))
    sortbyNumberAllelesMedian=sorted(allNumberAllelesMedian, key=itemgetter(1))

    #sortbyNumberAlleles.reverse()





    orderedlistgene=[]

    for elem in sortbymedia:
        elem.pop(-1)
        elem.pop(-1)
        orderedlistgene.append(elem.pop(-1))


    orderedlistgene2=[]
    sortbyNumberAllelesx=[]
    for elem in sortbyNumberAlleles:
        #orderedlistgene2.append(os.path.basename(elem.pop(0)))
        orderedlistgene2.append(os.path.join(relpath,(os.path.basename(elem.pop(0))).replace(".fasta",".html")))
        sortbyNumberAllelesx.append(elem.pop(0))

    orderedlistgene3=[]
    sortbyNumberAllelesMeanx=[]
    for elem in sortbyNumberAllelesMean:
        #orderedlistgene3.append(os.path.basename(elem.pop(0)))
        orderedlistgene3.append(os.path.join(relpath,(os.path.basename(elem.pop(0))).replace(".fasta",".html")))
        sortbyNumberAllelesMeanx.append(elem.pop(0))

    orderedlistgene4=[]
    sortbyNumberAllelesMedianx=[]
    for elem in sortbyNumberAllelesMedian:
        #orderedlistgene4.append(os.path.basename(elem.pop(0)))
        orderedlistgene4.append(os.path.join(relpath,(os.path.basename(elem.pop(0))).replace(".fasta",".html")))
        sortbyNumberAllelesMedianx.append(elem.pop(0))



    print(str(len(conservedlengthgenes)) +" conserved genes")
    print(str(len(notconservedlengthgenes)) +" not conserved genes")


    print("\nBuilding box plot...")

    if not ReturnValues	:
        with open ("notconserved.txt","w") as f:
            for gene in notconservedlengthgenes:
                f.write(str(gene)+"\n")

    else:

        print("Creating the allele number plot")

        orderedlistgene2_basename=[]
        for elem in orderedlistgene2:
            orderedlistgene2_basename.append(os.path.basename(elem))

        orderedlistgene2_html=[]
        for elem in orderedlistgene2:
            orderedlistgene2_html.append(os.path.join(relpath,(os.path.basename(elem)).replace(".fasta",".html")))


        sortbyNumberAlleles=[item for sublist in sortbyNumberAlleles for item in sublist]
        sortbyNumberAllelesMean=[item for sublist in sortbyNumberAllelesMean for item in sublist]
        sortbyNumberAllelesMedian=[item for sublist in sortbyNumberAllelesMedian for item in sublist]

        #numberallelesplotMedianhtml=[[sortbyNumberAllelesx,sortbyNumberAlleles,orderedlistgene2],[sortbyNumberAllelesMeanx,sortbyNumberAllelesMean,orderedlistgene3],[sortbyNumberAllelesMedianx,sortbyNumberAllelesMedian,orderedlistgene4]]
        numberallelesplotMedianhtml=[[sortbyNumberAllelesx,sortbyNumberAlleles,orderedlistgene2],[sortbyNumberAllelesMeanx,sortbyNumberAllelesMean,orderedlistgene3],[sortbyNumberAllelesMedianx,sortbyNumberAllelesMedian,orderedlistgene4]]


        print("Creating the allele size histogram")

        with open('locus_stats.tsv', "w") as f:
            f.write("Locus\tMode_value\tnumber_alleles\n")
            for elem2 in list(zip(genesList, modaStats,allAllelesStats)):
                f.write(os.path.basename(elem2[0])+"\t"+str(elem2[1])+"\t"+str(elem2[2])+"\n")

        histplothtml=modaStats

        finalList=[]
        finalNameList=[]
        finalLinkList=[]
        subList=[]
        nameSublist=[]
        linkSublist=[]
        i=0
        j=0
        os.path.join(relpath,(os.path.basename(elem)).replace(".fasta",".html"))
        while j<len(sortbymedia):

            if i >= split_thresh:
                #print len(subList)
                finalList.append(subList)
                subList=[]
                finalNameList.append(nameSublist)
                nameSublist=[]
                finalLinkList.append(linkSublist)
                linkSublist=[]
                i=0
                j-=1
            else:
                subList.append(sortbymedia[j])
                nameSublist.append(os.path.basename(orderedlistgene[j]))
                linkSublist.append(os.path.join(relpath,(os.path.basename(orderedlistgene[j])).replace(".fasta",".html")))
                i+=1
            j+=1

        if len(subList)>0:
            finalList.append(subList)
            finalNameList.append(nameSublist)
            finalLinkList.append(linkSublist)

        orderedlistgene=finalNameList
        boxplothtml=finalList

        return notconservedlengthgenes,len(genesList),genesWoneAllele,boxplothtml,histplothtml,numberallelesplotMedianhtml,orderedlistgene,j,finalLinkList,allAllelesStats

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import csv
import argparse


def main(mainListFile,toRemoveListFile,outputfileName,inverse):
    #~ parser = argparse.ArgumentParser(description="This program removes gens from a tab separated allele profile file")
    #~ parser.add_argument('-i', nargs='?', type=str, help='main matrix file from which to remove', required=True)
    #~ parser.add_argument('-g', nargs='?', type=str, help='list of genes to remove', required=True)
    #~ parser.add_argument('-o', nargs='?', type=str, help='output file name', required=True)
    #~ parser.add_argument("--inverse", help="list to remove is actually the one to keep", dest='inverse',
    #~ action="store_true", default=False)
    #~
    #~ args = parser.parse_args()
    #~ mainListFile = args.i
    #~ toRemoveListFile = args.g
    #~ outputfileName = args.o
    #~ inverse = args.inverse

    if inverse:
        FilesToRemove = ['File', 'FILE', 'file']
    else:
        FilesToRemove = []
    with open(toRemoveListFile) as f:
        for File in f:
            File = File.rstrip('\n')
            File = File.rstrip('\r')
            File = (File.split('\t'))[0]
            FilesToRemove.append(File)
    # print FilesToRemove

    with open(mainListFile, 'r') as tsvin, open(outputfileName + ".tsv", "w") as csvout:
        tsvin = csv.reader(tsvin, delimiter='\t')

        listindextoremove = []
        for firstline in tsvin:
            for gene in firstline:
                if gene in FilesToRemove and not inverse:
                    listindextoremove.append(firstline.index(gene))
                elif gene not in FilesToRemove and inverse:
                    listindextoremove.append(firstline.index(gene))

            for elem in reversed(listindextoremove):
                del firstline[elem]
            csvout.write(('\t'.join(firstline)) + "\n")
            break

        for line in tsvin:
            for elem in reversed(listindextoremove):
                del line[elem]
            csvout.write(('\t'.join(line)) + "\n")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import numpy as np
import argparse
from collections import Counter
import time
import plotly
import plotly.graph_objs as go
import os


def presAbs(d2c):
    print("generating presence abscence matrix")
    # genelist= d2[:1,1:]
    # print "Warning: all profiles must have same length"
    # for item in d2c:
    # print "genome "+str(item[0])+" has a profile length of "+str(len(item))
    geneslist = d2c[:1, :]
    genomeslist = d2c[1:, :1]

    # d2c = d2c[1:,:]

    row = 1
    while row < d2c.shape[0]:
        column = 1
        while column < d2c.shape[1]:
            try:

                aux = int(d2c[row, column])
                if aux > 0:
                    d2c[row, column] = 1
                else:
                    d2c[row, column] = 0
            except:
                try:
                    aux = str((d2c[row, column]).decode("utf-8")).replace("INF-", "")
                    aux = int(aux)
                    d2c[row, column] = 1
                except Exception as e:
                    d2c[row, column] = 0

            column += 1
        row += 1
    print("done")

    return d2c


# function to report the most problematic locus/genomes and report the number of good locus by % of genomes
# a list of genomes with a sum of problems higher than the given ythreshold will be returned
def presence3(d2, ythreshold, vector, abscenceMatrix,verbose):

    if verbose:
        def verboseprint(*args):
            for arg in args:
                print(arg, end="")
            print()
    else:
        verboseprint = lambda *a: None  # do-nothing function

    verboseprint( "checking loci with missing data ...")
    d2d = d2

    genomeslist = d2d[1:, :1]
    geneslist = d2d[0]

    # d2d = d2d[1:,1:]

    column = 1
    totals = []
    allbadgenomes = []
    reallybadgenomes = []
    allverygood = []
    allverybad = []
    plus95 = 0
    plus99 = 0
    plus995 = 0
    badGenomesScoreDict = {}
    listgenes2show = []

    # if a locus has one problematic call, check how many genomes have problem in that call

    while column < d2d.shape[1]:
        row = 1
        notfound = 0
        badgenomes = []
        while row < d2d.shape[0]:

            if not abscenceMatrix:

                if "LNF" in d2d[row, column] or d2d[row, column] == "NIPH" or "LOT" in d2d[row][column] or "ALM" in \
                        d2d[row][column] or "ASM" in d2d[row][column] or "ABM" in d2d[row][column] or "ERROR" in \
                        d2d[row][column] or "undefined" in d2d[row][column] or "small match" in d2d[row][
                    column] or "allele incomplete" in d2d[row][column]:

                    # if d2d[row,column] == "LNF" :
                    d2d[row, column] = 0
                    notfound += 1
                    badgenomes.append(row - 1)

                else:
                    d2d[row, column] = 1
                    pass
            else:
                if int(d2d[row, column]) == 0:
                    notfound += 1
                    badgenomes.append(row - 1)

                else:
                    d2d[row, column] = 1
                    pass

            row += 1

        value = float(d2d.shape[0] - notfound) / float(d2d.shape[0])

        if len(genomeslist) > 500:
            xthreshold = 0.95
        elif len(genomeslist) < 20:
            xthreshold = 0.90
        else:
            xthreshold = 0.95

        if value > xthreshold:
            listgenes2show.append(geneslist[column].decode("utf-8"))

        if value > xthreshold and value < 1:
            for badgenome in badgenomes:
                allbadgenomes.append((genomeslist[int(badgenome)])[0])
        elif value == 1:
            allverygood.append(geneslist[column])
        elif value == 0:
            allverybad.append(geneslist[column])
        if value >= 0.95:
            plus95 += 1
        if value >= 0.99:
            plus99 += 1
        if value >= 0.995:
            plus995 += 1

        totals.append(float(float(d2d.shape[0] - notfound) / float(d2d.shape[0])))
        column += 1

    counter = Counter(allbadgenomes)

    for elem in counter.most_common():
        if int(elem[1]) > ythreshold:
            reallybadgenomes.append((elem)[0])

    verboseprint('')
    verboseprint( "number genes in 100% genomes: " + str(len(allverygood)))
    verboseprint( "number genes above 99%: " + str(plus99))
    verboseprint( "number genes above 95%: " + str(plus95))
    verboseprint( "number genes above 99.5%: " + str(plus995))
    verboseprint( "number genes in 0% genomes: " + str(len(allverybad)))
    verboseprint( len(totals) )

    d2d = d2d.T

    # number of used genomes
    vector[0].append(len(genomeslist))

    # number of loci at 95%
    vector[1].append(plus95)

    # number of loci at 99%
    vector[2].append(plus99)

    # number of loci at 100%
    vector[6].append(len(allverygood))

    # number of loci at 0%
    vector[4].append(len(allverybad))

    # number of to be removed genomes%
    try:
        vector[5].append(len(reallybadgenomes) + ((vector[5])[-1]))
    except:
        vector[5].append(len(reallybadgenomes))

    # number of loci at 99.5%
    vector[3].append(plus995)

    verboseprint( "checked loci with missing data" )

    return d2d, reallybadgenomes, vector, True, listgenes2show


# ~ return d2d,reallybadgenomes,vector,True



def removegenomes(d2a, bagenomeslist,verbose):

    if verbose:
        def verboseprint(*args):
            for arg in args:
                print(arg, end="")
            print()
    else:
        verboseprint = lambda *a: None  # do-nothing function

    rowid = 1
    deleted = 0
    genomesList = (d2a[1:, :1]).tolist()

    verboseprint( len(genomesList))
    verboseprint( len(bagenomeslist))
    verboseprint( "removing genomes...")
    badgenomeslist2 = []
    for elem in bagenomeslist:
        badgenomeslist2.append(elem)

    if len(bagenomeslist) > 0:
        i = 0
        for genome in genomesList:
            i += 1
            if genome[0] in badgenomeslist2:
                d2a = np.delete(d2a, i, axis=0)
                deleted += 1
                i -= 1

    verboseprint( "genomes removed")
    verboseprint( d2a.shape)
    return d2a


def clean(d2, iterations, ythreshold,out_folder,verbose):

    if verbose:
        def verboseprint(*args):
            for arg in args:
                print(arg, end="")
            print()
    else:
        verboseprint = lambda *a: None  # do-nothing function

    abscencematrix = True

    i = 0
    removedlistgenomes = []
    toremovegenomes = []

    statsvector = [[] for x in range(7)]

    # run a function that gives information about the usable locus and the possible bad genomes
    lastremovedgenomesCount = 0
    iterationStabilizedat = None
    isStable = False
    listgenes2show = []
    listgenes2showtotal = []

    while i <= iterations:

        for elem in toremovegenomes:
            removedlistgenomes.append(elem.decode("utf-8"))

        if len(removedlistgenomes) > lastremovedgenomesCount and i > 0:
            lastremovedgenomesCount = len(removedlistgenomes)
        elif iterationStabilizedat is None and i > 0:
            iterationStabilizedat = i
            verboseprint( "stabilized at " + str(i))
            isStable = True
            listgenes2showtotal.append(listgenes2show)
        if not isStable:
            verboseprint( "\n########## ITERATION NUMBER %s  ##########  \n" % str(i) )
            verboseprint( "total removed genomes :" + str(len(removedlistgenomes)))
            d2 = removegenomes(d2, toremovegenomes,verbose)
            matrix3, toremovegenomes, statsvector, abscencematrix, listgenes2show = presence3(d2, ythreshold,
                                                                                              statsvector,
                                                                                              abscencematrix,verbose)
        # ~ matrix3,toremovegenomes,statsvector,abscencematrix=presence3(d2,ythreshold,statsvector,abscencematrix)

        else:
            for vector in statsvector:
                vector.append(vector[-1])

        i += 1

    with open(os.path.join(out_folder,"removedGenomes.txt"), "a") as f:
        f.write( (str(ythreshold) +"\t"+ (' '.join(map(str, removedlistgenomes)))+ "\n"))


    with open(os.path.join(out_folder,"Genes_95%.txt"), "a") as f:
        f.write(str(ythreshold) +"\t")

        for x in listgenes2showtotal:
            # ~ f.write(str(x)+"\n")
            f.write((' '.join(map(str, x))) + "\n")

    return statsvector, iterationStabilizedat


def main(pathOutputfile,iterationNumber,thresholdBadCalls,step,out_folder,verbose):


    #~ parser = argparse.ArgumentParser(
    #~ description="This program analyze an allele call raw output matrix, returning info on which genomes are responsible for cgMLST loci loss")
    #~ parser.add_argument('-i', nargs='?', type=str, help='raw allele call matrix file', required=True)
    #~ parser.add_argument('-n', nargs='?', type=int, help='maximum number of iterations', required=True)
    #~ parser.add_argument('-t', nargs='?', type=int, help='maximum threshold of bad calls above 95 percent', required=True)
    #~ parser.add_argument('-s', nargs='?', type=int, help='step between each threshold analysis', required=True)
    #~ parser.add_argument('-o', nargs='?', type=str, help="Folder for the analysis files", required=False, default=".")
    #~ parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
    #~ default=False)
    #~
    #~ args = parser.parse_args()
    #~
    #~ pathOutputfile = args.i
    #~ iterationNumber = int(args.n)
    #~ thresholdBadCalls = int(args.t)
    #~ step = int(args.s)
    #~ out_folder = args.o
    #~ verbose = args.verbose

    if verbose:
        def verboseprint(*args):
            for arg in args:
                print(arg, end="")
            print()
    else:
        verboseprint = lambda *a: None  # do-nothing function


    if not os.path.exists(out_folder):
        os.makedirs(out_folder)


    starttime = "\nStarting Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y")

    allresults = []
    threshold = 0
    thresholdlist = []
    listStableIter = []

    # for each threshold run a clean function on the dataset, using the previous run output (to be removed genomes) as input for the new one

    with open(os.path.join(out_folder,"removedGenomes.txt"), "w") as f:
        f.write( "Threshold\tRemoved_genomes\n")
    with open(os.path.join(out_folder,"Genes_95%.txt"), "w") as f:
        f.write( "Threshold\tPresent_genes\n")

    print("will try to open file...")
    # d2=np.loadtxt(inputfile, delimiter='\t')
    d2original = np.genfromtxt(pathOutputfile, delimiter='\t', dtype=None)
    d2copy = np.copy(d2original)
    # create abscence/presence matrix for easier processing
    d2copy = presAbs(d2copy)
    print("file was read")

    while threshold < thresholdBadCalls:
        thresholdlist.append(threshold)
        print(" ######## CALCULATING WITH THRESHOLD AT " + str(threshold) + " ########")
        result, stabilizedIter = clean(d2copy, iterationNumber, threshold,out_folder,verbose)
        listStableIter.append(stabilizedIter)
        allresults.append(result)

        threshold += step

    x = list(range(0, len(allresults)))

    i = 0

    labels = ["number of genomes",
              "Number of Loci present in 95% genomes",
              "Number of Loci present in 99% genomes",
              "Number of Loci present in 99.5% genomes",
              # "Number of Loci present in 0% genomes",
              # "Selected genomes to be removed",
              "Number of Loci present in 100% genomes"]

    i = iterationNumber
    while i <= iterationNumber:
        threshindex = 0

        aux2 = []
        for resultPerThresh in allresults:
            aux = []

            for result in resultPerThresh:
                aux.append(result[i])
            aux2.append(aux)

        d2 = np.asarray(aux2)

        d2 = d2.T
        linenmbr = 0

        d2 = np.delete(d2, (4), axis=0)
        d2 = np.delete(d2, (4), axis=0)

        listtraces = []
        for line in d2:

            if (linenmbr == 0):
                trace = go.Scatter(
                    x=thresholdlist,
                    y=line,
                    name="Number of genomes used",
                    mode='lines+markers',
                    yaxis='y2',
                    marker=dict(symbol='diamond-dot', size=10)

                )


            else:
                trace = go.Scatter(
                    x=thresholdlist,
                    name=labels[linenmbr],
                    mode='lines+markers',
                    y=line,
                    marker=dict(symbol='star-dot', size=10),
                    line=dict(dash='dash')
                )

            listtraces.append(trace)
            linenmbr += 1

        layout = go.Layout(
            title='Test genomes quality',
            xaxis=dict(
                title='Threshold'
            ),
            yaxis=dict(
                title='Number of loci'
            ),
            yaxis2=dict(
                title='Number of genomes',
                # ~ titlefont=dict(
                # ~ color='rgb(148, 103, 189)'
                # ~ ),
                # ~ tickfont=dict(
                # ~ color='rgb(148, 103, 189)'
                # ~ ),
                overlaying='y',
                side='right'
            )
        )

        fig = go.Figure({"data": listtraces, "layout": layout})
        plot_url = plotly.offline.plot(fig, filename=os.path.join(out_folder,'GenomeQualityPlot.html'))

        i += 1

    i = 0
    for stableiter in listStableIter:
        verboseprint( "At threshold " + str(thresholdlist[i]) + " it stabilized at the iteration number " + str(stableiter) )
        i += 1

    print (starttime)
    print ("Finished Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import sys
import os


def main(questionDB,directory,genomeFile,nucleotide):

    name = directory + "/" + genomeFile + "_db"

    if nucleotide==True:
        os.system("makeblastdb -in " + questionDB + " -out " + name + " -dbtype nucl -logfile " + name + "_blast.log")
    else:
        os.system("makeblastdb -in " + questionDB + " -out " + name + " -dbtype prot -logfile " + name + "_blast.log")

    return True


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from Bio.Blast import NCBIXML
import os


def ensure_dir(f):
    if not os.path.isdir(f):
        # print "No BLAST db dir was found. Creating it..."
        os.makedirs(f)


# overwrite has to be 1 or 0 (True or False)
def Create_Blastdb(questionDB, overwrite, dbtypeProt):
    base = os.path.basename(questionDB)
    dirname = os.path.dirname(questionDB)
    isProt = dbtypeProt

    if len(dirname) == 0:
        dirname = '.'
    basename = os.path.splitext(base)[0]
    ensure_dir(dirname + "/blastdbs")
    name = dirname + "/blastdbs/" + basename + "_db"

    if not os.path.isfile(name + ".nin") and not os.path.isfile(name + ".nhr") and not os.path.isfile(name + ".nsq"):

        if not isProt:
            os.system(
                "makeblastdb -in " + questionDB + " -out " + name + " -dbtype nucl -logfile " + name + "_blast.log")
        else:
            os.system(
                "makeblastdb -in " + questionDB + " -out " + name + " -dbtype prot -logfile " + name + "_blast.log")

    elif overwrite:
        if not isProt:
            os.system(
                "makeblastdb -in " + questionDB + " -out " + name + " -dbtype nucl -logfile " + name + "_blast.log")
        else:
            os.system(
                "makeblastdb -in " + questionDB + " -out " + name + " -dbtype prot -logfile " + name + "_blast.log")

    else:
        print("BLAST DB files found. Using existing DBs..")
    return name


def runBlast(cline, bOutFile, locus_sbjct):
    os.system(str(cline))
    rec = open(bOutFile)
    blast_record = NCBIXML.read(rec)

    if os.path.isfile(locus_sbjct):
        os.remove(locus_sbjct)
    os.remove(bOutFile)

    return blast_record


def runBlastParser(cline, bOutFile):
    os.system(str(cline))
    rec = open(bOutFile)
    blast_records = NCBIXML.parse(rec)

    #	if os.path.isfile(locus_sbjct):
    #		os.remove(locus_sbjct)

    # os.remove(bOutFile)

    return blast_records



#!/usr/bin/env python3
import os
import argparse
import json
from operator import itemgetter
from Bio import SeqIO
from Bio.Alphabet import generic_dna
import time
import sys

try:
	from SchemaEvaluator import CheckCDS,alleleSizeStats
except ImportError:
	from CHEWBBACA.SchemaEvaluator import CheckCDS,alleleSizeStats

def which(program):
    import os
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return True
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return True
    print(program+" not found")
    sys.exit()
    return "Not found"

def main(genes,cpuToUse,htmlFile,transTable,threshold,splited,title,logScale,OneBadGeneNotConserved,skipHeavy):
    #~ parser = argparse.ArgumentParser(
        #~ description="This program analyses a set of gene files, analyzing the alleles CDS and the length of the alleles per gene")
    #~ parser.add_argument('-i', nargs='?', type=str, help='list genes, directory or .txt file with the full path',
                        #~ required=True)
    #~ parser.add_argument('-p', nargs='?', type=bool, help='One bad allele still makes gene conserved', required=False,
                        #~ default=False)
    #~ parser.add_argument('--log', dest='logScale', action='store_true')
    #~ parser.add_argument('-l', nargs='?', type=str, help='name/location main html file', required=True)
    #~ parser.add_argument('-ta', nargs='?', type=int, help='ncbi translation table', required=False, default=11)
    #~ parser.add_argument('-t', nargs='?', type=float, help='Threshold', required=False, default=0.05)
    #~ parser.add_argument('--title', nargs='?', type=str, help='title on the html', required=False,
                        #~ default="My Analyzed wg/cg MLST Schema - Rate My Schema")
    #~ parser.add_argument('--cpu', nargs='?', type=int, help='number of cpu to use', required=True)
    #~ parser.add_argument('-s', nargs='?', type=int,
                        #~ help='number of boxplots per page (more than 500 can make the page very slow)', required=False,
                        #~ default=500)
    #~ parser.add_argument('--light', help="skip clustal and mafft run", required=False, action="store_true", default=False)
    #~ parser.set_defaults(logScale=False)
#~ 
    #~ args = parser.parse_args()
    #~ genes = args.i
    #~ transTable = args.ta
    #~ logScale = args.logScale
    #~ htmlFile = args.l
    outputpath = os.path.dirname(htmlFile)
    #~ cpuToUse = args.cpu
    #~ threshold = float(args.t)
    #~ OneBadGeneNotConserved = bool(args.p)
    #~ splited = int(args.s)
    #~ title = str(args.title)
    #~ skipHeavy = args.light

    starttime = "\nStarting Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y")
    print (starttime)
	
    print ("Checking all programs are installed")
    print ("Checking mafft installed... " + str(which('mafft')))
    print ("Checking clustalw2 installed... " + str(which('clustalw2')))
	
	
    try:
        f = open(genes, 'r')
        f.close()
    except IOError:
        listbasename = os.path.basename(os.path.normpath(genes))

        with open("listGenes" + listbasename + ".txt", "w") as f:
            for gene in os.listdir(genes):
                try:
                    genepath = os.path.join(genes, gene)
                    #gene_fp2 = HTSeq.FastaReader(genepath)
                    for allele in SeqIO.parse(genepath, "fasta", generic_dna):
                        break
                    f.write(genepath + "\n")
                except Exception as e:
                    print(e)
                    pass

        genes = "listGenes" + listbasename + ".txt"

    genebasename = str(os.path.basename(genes))
    genebasename = genebasename.split(".")
    genebasename.pop()
    genebasename = ".".join(genebasename)
    # genebasename=genebasename[0]


    notConservedgenes, totalgenes, genesWOneAllele, boxplot, histplot, allelenumberplot, listgenesBoxOrdered, totalnumberofgenes, boxListLink, allAllelesStats = alleleSizeStats.main(
        genes, threshold, OneBadGeneNotConserved, True, logScale, outputpath, splited)

    # boxplot=str(json.dumps(boxplot))
    histplot = str(json.dumps(histplot))
    allelenumberplot = str(json.dumps(allelenumberplot))
    allAllelesStats = str(json.dumps(allAllelesStats))

    statsPerGene = CheckCDS.main(genes, transTable, True, outputpath, cpuToUse,skipHeavy)

    # stats values are ordered in a list allelesNotMultiple3,listStopcodonsInside,listnotStartCodon,numberOfAlleles

    htmlgenespath = os.path.join(outputpath, "genes_html/")
    relpath = os.path.relpath(htmlgenespath, outputpath)

    if not os.path.exists(htmlgenespath):
        os.makedirs(htmlgenespath)

    with open(htmlFile, "w") as f:
        f.write(
            "<!DOCTYPE html>\n<html>\n<head><script type='text/javascript' src='https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js'></script>\n")
        f.write(
            "<script type='text/javascript' src='https://ajax.googleapis.com/ajax/libs/jquery/2.1.3/jquery.min.js'></script>")
        f.write("""<!-- Latest compiled and minified JavaScript -->
		<script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
<script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/js/bootstrap.min.js" integrity="sha384-0mSbJDEHialfmuBBQP6A4Qrprq5OVfW37PRR3j5ELqxss1yVqOtnepnHVP9aJ7xS" crossorigin="anonymous"></script>""")
        f.write("""<style type="text/css">
      body {
        padding-bottom: 60px;
        padding-left: 20px;
        padding-right: 20px;
      }

      /* Custom container */
      .container {
        margin: 0 auto;
      }
      .container > hr {
        margin: 60px 0;
      }

      /* Main marketing message and sign up button */
      .jumbotron {
        text-align: center;
      }
      .jumbotron h1 {
        font-size: 100px;
        line-height: 1;
      }
      .jumbotron .lead {
        font-size: 24px;
        line-height: 1.25;
      }
      .jumbotron .btn {
        font-size: 21px;
        padding: 14px 24px;
      }

    </style>
		<!-- Latest compiled and minified CSS -->
<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css" integrity="sha384-1q8mTJOASx8j1Au+a5WDVnPi2lkFfwwEAa8hDDdjZlpLegxhjVME1fgjWPGmkzs7" crossorigin="anonymous">
<script>
function openList1(documentid) {
    var list = document.getElementById(documentid);

    if (list.style.display == "none"){
        list.style.display = "block";
    }else{
        list.style.display = "none";
    }
}
</script>""")

        f.write("""<style type="text/css">
		ul {
    /*min-height: 300px;*/
    -webkit-column-count: 4;
       -moz-column-count: 4;
            column-count: 4; /*4 is just placeholder -- can be anything*/
}
li {
    display: table;
    padding-bottom: 20px; 
    margin-right: 30px;
}
li a {
    color: rgb(0, 162, 232);
}

.tg  {border-collapse:collapse;border-spacing:0;border-color:#999;}
.tg td{font-family:Arial, sans-serif;font-size:14px;padding:10px 5px;border-style:solid;border-width:1px;overflow:hidden;word-break:normal;border-color:#999;color:#444;background-color:#F7FDFA;}
.tg th{font-family:Arial, sans-serif;font-size:14px;font-weight:normal;padding:10px 5px;border-style:solid;border-width:1px;overflow:hidden;word-break:normal;border-color:#999;color:#fff;background-color:#26ADE4;}
.tg .tg-vn4c{background-color:#D2E4FC;text-align:center}
.tg .tg-qpvr{background-color:#009901;vertical-align:top}
.tg .tg-14d4{background-color:#91df0a;text-align:center;vertical-align:top}

</style>\n</head>\n<body>""")

        f.write("""<div class="jumbotron">
  <h2>""" + title + """</h2>
  <p>Explore the analysis by clicking the analysis buttons</p></div>""")

        f.write("<h2>Allele size analysis using a mode +/- " + str(threshold) + "</h2>")
        f.write(
            "<p> Genes are considered not conserved if >1 allele are outside the mode +/-0.05 size. Genes with only 1 allele outside the threshold are considered conserved</p>\n")
        f.write("<h3>" + str(totalgenes) + " total loci</h3>")
        f.write("\n<h3>" + str(len(notConservedgenes)) + " loci with high length variability</h3>")
        f.write(
            """<button onclick = "openList1('ollist1')">Show/hide list</button><ol id='ollist1' style='display: none;'>""")

        for elem in notConservedgenes:
            f.write("<li><a href = '" + str(elem) + "' target='_blank'>" + (os.path.basename(elem)).replace(".html",
                                                                                                            "") + "</a></li>")

        f.write("</ol>\n<h3>" + str(len(genesWOneAllele)) + " loci with only one allele</h3>\n")

        f.write(
            """<button onclick = "openList1('ollist2')">Show/hide list</button><ol id='ollist2' style='display: none;'>""")

        for elem in genesWOneAllele:
            f.write("<li><a href = '" + str(elem) + "' target='_blank'>" + (os.path.basename(elem)).replace(".html",
                                                                                                            "") + "</a></li>\n")

        f.write("</ol>\n")

        f.write("""<div class='container'>
					<div class='row'>
						<div class="col-sm-3">
						<button id='button2' class="btn btn-success btn-block active">Allele length analysis</button>
						</div>
						<div class="col-sm-3">
						<button id='button3' class="btn btn-success btn-block active">Allele number analysis</button>
						</div>
						<div class="col-sm-3">
						<button id='button1' class="btn btn-success btn-block active">Loci size variation analysis</button>
						</div>
						<div class="col-sm-3">
						<button id='button4' class="btn btn-success btn-block active">CDS analysis</button>
						</div>
					</div>
				</div>""")

        f.write("""<div id="fig03" ><div id="fig03b" style="width: 1000px; height: 600px;"></div><div id="fig03a" style="width: 1300px; height: 700px;"></div></div>
		<div id="fig01" style="display:none; width: 100%;">
		<h2>Size boxplot for all loci</h2><p>Box plot for the sizes of each locus. Loci ordered by increasing median allele size</p>
		<p>Use the zoom button and hover the mouse over a box/median to see the locus name and points data</p>
		<p>-->Use the following buttons to navigate through all loci</p>
		<button id='buttonbackward' > Previous 500 loci < </button>
		<button id='buttonforward' > Next 500 loci > </button>
		""")

        f.write("""<div id="figbox0" classe="container" style="display:none;width: 1500px; height: 1000px;"></div>""")

        f.write(
            """</div><div id="fig02" classe="container" style="display:none;width: 1000px; height: 600px;"></div>""")

        f.write("""<script type="text/javascript">
					
					$("#buttonforward").click(function(){
					  boxplotPage=boxplotPage+1;
					  $.ajax({
									async: false,
									url: "jsonbox"+boxplotPage+".js",
									dataType: "script",
									
									error: function(){
										  boxplotPage=boxplotPage-1;
											},
								  success: function(){
						  										
										numberofgenes=numberofgenes+jsonboxList.length
										var listboxes=[];
										var i = 0;
										var len = jsonboxList.length;
										for (; i < len; i++) { 
											var trace = {
														  y: jsonboxList[i],
														  name: listgenes[i],
														  text: listlinks[i],
														  type: 'box',
														  marker: {size: 2},
														};
											listboxes.push(trace)
										}
										var layout = {
													  title: numberofgenes+'/'+totalGenes,
													  showlegend: false,
													  xaxis: {
																autotick: false,
																showgrid: true,
																showticklabels: false
															  },
													};
										Plotly.newPlot('figbox0', listboxes,layout);
						
						var myPlot = document.getElementById('figbox0');
							myPlot.on('plotly_click', function(data){
								link = '';
								for(var i=0; i < data.points.length; i++){
									link = data.points[i].data.text;
								}
								window.open(link, '_blank');
							});
						
						}
					    
					});
					return false;
					}); 
					</script>""")

        f.write("""<script type="text/javascript">
					$("#buttonbackward").click(function(){
					  boxplotPage=boxplotPage-1;
					  numberofgenes=numberofgenes-jsonboxList.length;
					  $.ajax({
									async: false,
									url: "jsonbox"+boxplotPage+".js",
									dataType: "script",
									
									error: function(){
										numberofgenes=numberofgenes+jsonboxList.length;
										  boxplotPage=boxplotPage+1;
											},
								  success: function(){
						  										
										var listboxes=[];
										var i = 0;
										var len = jsonboxList.length;
										for (; i < len; i++) { 
											var trace = {
														  y: jsonboxList[i],
														  name: listgenes[i],
														  text: listlinks[i],
														  type: 'box',
														  marker: {size: 2},
														};
											listboxes.push(trace)
										}
										var layout = {
													  title: numberofgenes+'/'+totalGenes,
													  showlegend: false,
													  xaxis: {
																autotick: false,
																showgrid: true,
																showticklabels: false
															  },
													};
										Plotly.newPlot('figbox0', listboxes,layout);
						
						var myPlot = document.getElementById('figbox0');
							myPlot.on('plotly_click', function(data){
								link = '';
								for(var i=0; i < data.points.length; i++){
									link = data.points[i].data.text;
								}
								window.open(link, '_blank');
							});
						
						}
					    
					});
					return false;
					}); 
					</script>""")

        f.write("""<script type="text/javascript">
					$("#button3").click(function(){
					  if ($("#fig03a").firstChild==undefined){
					  $.ajax({
								async: false,
								url: "json3.js",
								dataType: "script"
							});
						var listTraces=[];
						var i = 0;
						var len = jsonsScatterPlot.length;
						var listName=['Mode','Mean','Median'];
						for (; i < len; i++) { 
							var aux=jsonsScatterPlot[i];
							var listgenes=[];
							var j = 0;
							for (; j < aux[2].length; j++) { 
								genename=aux[2][j];
								listgenes.push(genename.replace(/^.*[\\\/]/, ''));
							}
							var trace = {
									x: aux[0],
									  y: aux[1],
									  name: listName[i],
									  customdata: aux[2],
									  text: listgenes,
									  mode: 'markers',
									  type: 'scattergl'
									};
							listTraces.push(trace)
								}
						var layout = {
									  yaxis: {title: "Number of alleles"},
									  xaxis: {
												title: "Allele size in bp",
											  },
									};
						Plotly.newPlot('fig03a', listTraces,layout);
							
						var trace = [{
									  x: listNumberDifAlleles,
									  type: 'histogram'
									}];
						var layout = {
									  barmode: "stack",
									  showlegend: false,
									  yaxis: {title: "Number of Loci"},
									  xaxis: {
												autotick: true,
												showgrid: true,
												title: "Number of Different Alleles",
												showticklabels: true
											  },
									};
						Plotly.newPlot('fig03b', trace,layout);
						
						var myPlot = document.getElementById('fig03a');
							myPlot.on('plotly_click', function(data){
								link = '';
								link = data.points[0].data.customdata[data.points[0].pointNumber];
								window.open(link, '_blank');
							});
						
						
					  }
					  $("#fig03").css({"display":"block"});
					  $("#fig01").css({"display":"none"});
					  $("#fig02").css({"display":"none"});
					  $("#fig04").css({"display":"none"});
					}); 
					</script>""")

        f.write("""<script type="text/javascript">
					$("#button2").click(function(){
					 if ($("#fig02").firstChild==undefined){
						$.ajax({
								async: false,
								url: "json2.js",
								dataType: "script"
							});
						
	
						var trace = [{
									  x: jsonHistPlot,
									  type: 'histogram'
									}];
						var layout = {
									  title: 'Distribution of allele mode sizes per locus',
									  barmode: "stack",
									  showlegend: false,
									  yaxis: {title: "Number of locus"},
									  xaxis: {
												autotick: true,
												showgrid: true,
												title: "Allele Size",
												showticklabels: true
											  },
									};
						Plotly.newPlot('fig02', trace,layout);
					}
					  $("#fig02").css({"display":"block"});
					  $("#fig03").css({"display":"none"});
					  $("#fig01").css({"display":"none"});
					  $("#fig04").css({"display":"none"});
					}); 
					</script>""")

        f.write("""<script type="text/javascript">
					var boxplotPage=0;
					var numberofgenes=0
					var totalGenes=""" + str(totalnumberofgenes) + """
					$("#button1").click(function(){
					 if ($("#figbox0").firstChild==undefined){
						$.ajax({
								async: false,
								url: "jsonbox0.js",
								dataType: "script"
							});
						var listboxes=[];
						var i = 0;
						numberofgenes=jsonboxList.length;
						var len = jsonboxList.length;
						for (; i < len; i++) { 
							var trace = {
										  y: jsonboxList[i],
										  name: listgenes[i],
										  text: listlinks[i],
										  type: 'box',
										  marker: {size: 2},
										};
							listboxes.push(trace);
						}
						var layout = {
									  title: numberofgenes+'/'+totalGenes,
									  showlegend: false,
									  xaxis: {
												autotick: false,
												showgrid: true,
												showticklabels: false
											  },
									};
						Plotly.newPlot('figbox0', listboxes,layout);
						
						var myPlot = document.getElementById('figbox0');
							myPlot.on('plotly_click', function(data){
								link = '';
								for(var i=0; i < data.points.length; i++){
									link = data.points[i].data.text;
								}
								window.open(link, '_blank');
							});
						}
						
					  $("#fig01").css({"display":"block"});
					  $("#figbox0").css({"display":"block"});
					  $("#fig02").css({"display":"none"});
					  $("#fig03").css({"display":"none"});
					  $("#fig04").css({"display":"none"});
					}); 
					</script>""")
        f.write("""<script type="text/javascript">
					$("#button4").click(function(){
					  $("#fig04").css({"display":"block"});
					  $("#fig02").css({"display":"none"});
					  $("#fig03").css({"display":"none"});
					  $("#fig01").css({"display":"none"});
					}); 
					</script>""")

        f.write("""<div id="fig04" classe="container" style="display:none">
		<title>Schema Validation Results</title>
		<h3>Summary of problematic alleles per locus using the <a href='http://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi#SG11'>NCBI translation table 11</a></h3>
		<div id="fig04B" classe="container" style="width: 1200px; height: 700px;"></div>
		<p>Click the locus name to open the locus info page</p>
		<p>click the boxes with % to get the index of the alleles with problems</p>
		<table class="tg">
		  <tr>
			<th class="tg-031e">Gene</th>
			<th class="tg-031e">Number alleles not multiple of 3</th>
			<th class="tg-031e">Number alleles w/ >1 stop codons</th>
			<th class="tg-031e">Number alleles wo/ Start/Stop Codon</th>
			<th class="tg-qpvr" .background-color='#59b300'>Number of alleles (% alleles w/ issues) </th>
		  </tr>""")
        ordered = []
        orderedBySize = []
        for key, value in statsPerGene.items():
            numberMultip = float(len(value[0]))
            numberStop = float(len(value[1]))
            numberStart = float(len(value[2]))
            total = float(value[3])
            totalpercent = ((numberMultip + numberStart + numberStop) / total) * 100
            ordered.append([key, totalpercent])
            orderedBySize.append([key, total])
        ordered = sorted(ordered, key=itemgetter(-1))
        ordered.reverse()
        orderedBySize = sorted(orderedBySize, key=itemgetter(-1))

        newlist = []
        auxHistList = [[], [], [], [], []]
        i = 0
        while i < len(ordered):
            item = ordered[i]

            aux = []
            aux.append(item[0])

            value = statsPerGene[item[0]]
            numberMultip = float(len(value[0]))
            numberStop = float(len(value[1]))
            numberStart = float(len(value[2]))
            total = float(value[3])

            aux.append(value)
            newlist.append(aux)
            name = os.path.basename(str(item[0]))
            name = name.split(".")
            name = name[0]
            if (numberMultip > 0 or numberStop > 0 or numberStart > 0):
                locusHTML = os.path.join(relpath, (os.path.basename(str(item[0]))).replace(".fasta", ".html"))
                f.write("<tr id=" + str(item[
                                            0]) + """>\n<td class='tg-vn4c' onclick="window.open('""" + locusHTML + """')" style='cursor:pointer'>""" + name + "</td>\n<td class='tg-vn4c' onclick='a(this);'>" + str(
                    int(numberMultip)) + " (" + str('{0:.2f}'.format(
                    (numberMultip / total) * 100)) + "%)" + "</td>\n<td class='tg-vn4c' onclick='a(this);'>" + str(
                    int(numberStop)) + " (" + str('{0:.2f}'.format(
                    (numberStop / total) * 100)) + "%)" + "</td>\n<td class='tg-vn4c' onclick='a(this);'>" + str(
                    int(numberStart)) + " (" + str('{0:.2f}'.format(
                    (numberStart / total) * 100)) + "%)" + "</td>\n<td class='tg-14d4' onclick='a(this);'>" + str(
                    int(total)) + " (" + str('{0:.2f}'.format(
                    ((numberMultip + numberStart + numberStop) / total) * 100)) + "%)" + "</td>\n</tr>")
            i += 1
        i = 0
        while i < len(orderedBySize):
            item2 = orderedBySize[i]
            value2 = statsPerGene[item2[0]]

            if (len(value2[0]) > 0 or len(value2[1]) > 0 or len(value2[2]) > 0):
                locusHTML = os.path.join(relpath, (os.path.basename(str(item2[0]))).replace(".fasta", ".html"))
                auxHistList[0].append(locusHTML)
                auxHistList[1].append(float(len(value2[0])))
                auxHistList[2].append(float(len(value2[1])))
                auxHistList[3].append(float(len(value2[2])))
                auxHistList[4].append(
                    float(value2[3]) - float(len(value2[2])) - float(len(value2[1])) - float(len(value2[0])))
            i += 1

        auxHistList = str(json.dumps(auxHistList))
        f.write("</table>")
        f.write(
            """<div id='AllelesWissues'></div><button onclick="$('#AllelesWissues').empty();">clean</button></div>""")
        f.write("""\n<script type='text/javascript'>
		var histCDSanalysis=""" + auxHistList + """;
		var listTraces=[];
		var trace = {
					  x: histCDSanalysis[1],
					  name: 'Non multiple 3',
					  link: histCDSanalysis[0],
					  orientation: 'h',
					  type: 'bar'
					};
		var trace1 = {
					  x: histCDSanalysis[2],
					  name: '> 1 stop codon',
					  link: histCDSanalysis[0],
					  orientation: 'h',
					  type: 'bar'
					};
		var trace2 = {
					  x: histCDSanalysis[3],
					  name: 'No Start/Stop codon',
					  link: histCDSanalysis[0],
					  orientation: 'h',
					  type: 'bar'
					};
		var trace3 = {
					  x: histCDSanalysis[4],
					  name: 'CDS Alleles',
					  link: histCDSanalysis[0],
					  orientation: 'h',
					  type: 'bar'
					};			
		
					
		var layout = {
					  barmode: "stack",
					  yaxis: {
								autotick: false,
								showgrid: false,
								showticklabels: false
					  
							},
					  xaxis: {title: "Number of occurrences"},
								
					};
					
		listTraces.push(trace3,trace2,trace,trace1);			
		Plotly.newPlot('fig04B', listTraces,layout);
		
		var myPlot = document.getElementById('fig04B');
					myPlot.on('plotly_click', function(data){
						i= data.points[0].y
						link = data.points[0].data.link[i];
						window.open(link, '_blank');
					});
		
		</script>
		""")
        f.write("""\n<script type='text/javascript'>function a(element) {
	var id = $(element).closest("tr").attr("id");
	var badalleles=[];
	for (i = 0; i < alleles.length; i++) { 
		if((alleles[i])[0]==id){
			badalleles=(alleles[i])[1];
			break;
			}
		}
	var notmulti=(badalleles[0]).join('; ');
	var stopcodon=(badalleles[1]).join('; ');
	var startcodon=(badalleles[2]).join('; ');
	var name=(id.split("/")).slice(-1)[0]
	name=(name.split("."))[0]
	$('#AllelesWissues').append('<h2> Locus: '+name+'</h2>');
	$('#AllelesWissues').append('<p> Alleles not multiple of 3: '+notmulti+'</p><p> Alleles with >1 stop codon: '+stopcodon+'</p><p>Alleles without start or stop codon: '+startcodon+'</p>');

	$('html,body').animate({
        scrollTop: $('#AllelesWissues').offset().top},'slow');
	
	}
	</script>""")

        f.write("\n<script type='text/javascript'>var alleles=" + json.dumps(newlist) + "</script>")
        f.write("</body>\n</html>")

    i = 0

    filename = "jsonbox0.js"
    for elem in boxplot:
        boxplotElem = str(json.dumps(elem))
        listGenesJson = str(json.dumps(listgenesBoxOrdered[i]))
        listLinkGenesJson = str(json.dumps(boxListLink[i]))
        filename = "jsonbox" + str(i) + (".js")
        with open((os.path.join(outputpath, filename)), "w") as f:
            f.write("var jsonboxList =" + str(boxplotElem) + ";var listgenes=" + str(
                listGenesJson) + ";var listlinks=" + str(listLinkGenesJson))
        i += 1

    with open((os.path.join(outputpath, "json2.js")), "w") as f:

        f.write("var jsonHistPlot =" + str(histplot))

    with open((os.path.join(outputpath, "json3.js")), "w") as f:

        f.write("var jsonsScatterPlot =" + str(allelenumberplot) + ";var listNumberDifAlleles=" + allAllelesStats)

    try:
        os.remove("listGenes" + listbasename + ".txt")
    except:
        pass

    print (starttime)
    print ("Finished Script at : " + time.strftime("%H:%M:%S-%d/%m/%Y"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import os
import numpy as np
import pandas as pd


def presAbs(calls, listgenomesRemove, outputfile, cgPercent):

    def binarize(column):

        fix_inf = column.replace(to_replace='INF-', value='', regex=True)
        col = fix_inf.replace(to_replace='\D+.*', value='0', regex=True)
        coln = pd.to_numeric(col)

        return np.int64(coln > 0)

    def above_threshold(column, column_length):

        return (np.sum(column) / column_length) >= cgPercent

    # remove genomes
    for genome in calls.index:

        if genome in listgenomesRemove:

            print("removed genome : {}".format(genome))

    to_remove_bool = calls.index.isin(listgenomesRemove)

    calls_removed = calls.loc[~ to_remove_bool]

    print ("building the presence and absence matrix...")

    presence_absence = calls_removed.apply(binarize)


    pa_path = os.path.join(outputfile, 'Presence_Absence.tsv')
    presence_absence.to_csv(pa_path, sep='\t')

    print ("presence and abscence matrix built")

    pa_rows, _ = presence_absence.shape
    is_above_threshold = presence_absence.apply(above_threshold,
                                                args=(pa_rows,))

    columns_to_remove = calls_removed.columns[~ is_above_threshold]

    return presence_absence, calls_removed, columns_to_remove

def missing_data_table(cleaned_calls):

    n_genes = len(cleaned_calls.columns)
    genes_present = cleaned_calls.apply(np.count_nonzero, axis=1)

    missing_data = {'FILE': cleaned_calls.index,
                    'number of missing data': n_genes - genes_present,
                    'percentage': 1 - (genes_present / n_genes)}

    missing_data_df = pd.DataFrame(missing_data,
                                   columns=['FILE',
                                            'number of missing data',
                                            'percentage'])

    return missing_data_df


def clean(inputfile, outputfile, toremovegenes, toremovegenomes, cgPercent):

    # open the raw file to be clean
    calls = pd.read_csv(inputfile,
                        header=0, index_col=0, sep='\t', low_memory=False)

    # get presence abscence matrix
    presence_absence, calls_removed, cols_to_remove = presAbs(calls,
                                                              toremovegenomes,
                                                              outputfile,
                                                              cgPercent)


    total_genes_to_delete = set(toremovegenes).union(set(cols_to_remove))

    # clean the original matrix,
    # using the information on the presence/abscence matrix
    print ("processing the matrix")

    to_keep = ~calls_removed.columns.isin(total_genes_to_delete)

    cleaned = calls_removed.loc[:, to_keep]

    # remove INF and other missing data tags from the profile
    words = ['LNF', 'PLOT3', 'PLOT5', 'ASM', 'ALM', 'NIPHEM', 'NIPH', 'LOTSC']
    for word in words:
        cleaned = cleaned.replace(to_replace='{}.*'.format(word),
                                  value='0', regex=True)

    cleaned = cleaned.replace(to_replace='INF-', value='', regex=True)
    cleaned = cleaned.apply(pd.to_numeric)
    # cleaned = cleaned.reindex(cleaned.index.rename(['FILE']))

    # count number of missing data per genome
    missing_data_df = missing_data_table(cleaned)

    # write the output file
    cgmlst_path = os.path.join(outputfile, 'cgMLST.tsv')
    cleaned.to_csv(cgmlst_path, sep='\t')

    schema_path = os.path.join(outputfile, 'cgMLSTschema.txt')
    pd.Series(list(cleaned.index)).to_csv(schema_path, index=False)

    mdata_path = os.path.join(outputfile, 'mdata_stats.tsv')
    missing_data_df.to_csv(mdata_path, sep='\t', index=False)

    cgmlst_genes = len(cleaned.columns)
    totaldeletedgenes = len(calls.columns) - cgmlst_genes

    print ("deleted : {} loci".format(totaldeletedgenes))
    print ("total loci remaining : {}".format(cgmlst_genes))


def main(pathOutputfile,newfile,percent,genesToRemoveFile,genomesToRemoveFile):

    if not os.path.exists(newfile):
        os.makedirs(newfile)

    genesToRemove = []
    genomesToRemove = []

    if genesToRemoveFile:

        genesToRemove = pd.read_csv(genesToRemoveFile,
                                    header=None, index_col=None)[0]

    if genomesToRemoveFile:

        genomesToRemove = pd.read_csv(genomesToRemoveFile,
                                      header=None, index_col=None)[0]

    clean(pathOutputfile, newfile, genesToRemove, genomesToRemove, percent)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import csv
import numpy as np
from numpy import array
import argparse
import operator
from collections import Counter
import os

def main(contigsfile,out_folder):

    #~ parser = argparse.ArgumentParser(description="Check if locus are being represented more than once")
    #~ parser.add_argument('-i', nargs='?', type=str, help='contig info file', required=True)
    #~ parser.add_argument('-o', nargs='?', type=str, help="Folder for the analysis files", required=False,default=".")
    #~
    #~
    #~ args = parser.parse_args()
    #~ contigsfile = args.i
    #~ out_folder = args.o

    if not os.path.exists(out_folder):
        os.makedirs(out_folder)

    with open(contigsfile) as f:
        reader = csv.reader(f, delimiter="\t")
        d = list(reader)

    d2 = array(d)

    genelist= d2[:1,:]
    genelist=genelist.tolist()[0]
    genomeslist= d2[:,:1]
    genomeslist=genomeslist.tolist()

    i=1
    paralogs=[]
    pontuationDict={}
    pontuationDict2={}

    while i<len(genomeslist):

        genomeSchema2=d2[i]
        genomeSchema=genomeSchema2.tolist()

        i+=1

        #per each line(genome) get the contigs+positions that are repeated
        #remove the repeated elements that are not contigs+positions

        equalelem=[x for x, y in Counter(genomeSchema).items() if y > 1]

        if "LNF" in equalelem:
            equalelem.remove("LNF")
        if "LOT3" in equalelem:
            equalelem.remove("LOT3")
        if "LOT5" in equalelem:
            equalelem.remove("LOT5")
        if "LOTSC" in equalelem:
            equalelem.remove("LOTSC")
        if "PLOT3" in equalelem:
            equalelem.remove("PLOT3")
        if "PLOT5" in equalelem:
            equalelem.remove("PLOT5")
        if "PLOTSC" in equalelem:
            equalelem.remove("PLOTSC")
        if "PLOT" in equalelem:
            equalelem.remove("PLOT")
        if "NIPH" in equalelem:
            equalelem.remove("NIPH")
        if "NIPHEM" in equalelem:
            equalelem.remove("NIPHEM")
        if "ALM" in equalelem:
            equalelem.remove("ALM")
        if "ASM" in equalelem:
            equalelem.remove("ASM")
        if "allele incomplete" in equalelem:
            equalelem.remove("allele incomplete")
        if "undefined" in equalelem:
            equalelem.remove("ASM")
        if "small match" in equalelem:
            equalelem.remove("small match")


        #give a +1(paralog) point to a locus per each reapeated contig+position

        for elem in equalelem:

            ii = np.where(genomeSchema2 == elem)[0]

            for index in ii:
                paralogs.append(genelist[index])


                if genelist[index] in pontuationDict:
                    pontuationDict[genelist[index]]+=1
                else:
                    pontuationDict[genelist[index]]=1
        j=0

        #give a +1(problem) point to a locus per each reapeated contig+position
        for elem in genomeSchema:

            if elem == "LNF" or "LOT" in elem or "PLOT" in elem or elem == "NIPH" or elem == "NIPHEM" or elem == "ALM" or elem == "ASM" or elem == "allele incomplete" or elem == "undefined" or elem == "small match":
                if genelist[j] in pontuationDict2:
                    pontuationDict2[genelist[j]]+=1
                else:
                    pontuationDict2[genelist[j]]=1
            j+=1
    ordered = sorted(pontuationDict.items(), key=operator.itemgetter(1))

    paralogs=set(paralogs)
    print("Detected number of paralog loci: "+ str(len(paralogs)))


    #write file with a overrepresented locus per line, the number of times the locus is overrepresented, problems and total of overrepresentation+problems

    with open(os.path.join(out_folder,"RepeatedLoci.txt"), "w") as f:
        f.write("gene\tPC\tNDC\n")
        for k,v in ordered:
            try:
                troubledLocus=str(pontuationDict2[k])
            except:
                troubledLocus="0"
                pass
            f.write( (k+ "\t"+str( v)+"\t"+ troubledLocus+"\n"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
from Bio import SeqIO
from Bio.Alphabet import generic_dna
import os
import argparse


def get_Short(genesList):
    for gene in genesList:
        # gene = gene.rstrip('\n')
        pathtoDir = os.path.join(os.path.dirname(gene), "short")
        if not os.path.exists(pathtoDir):
            os.makedirs(pathtoDir)
        shortgene = os.path.join(os.path.dirname(gene), "short", os.path.basename(gene))
        shortgene = shortgene.replace(".fasta", "_short.fasta")

        #gene_fp2 = HTSeq.FastaReader(gene)
        for allele in SeqIO.parse(gene, "fasta", generic_dna):
            fG = open(shortgene, 'w')
            fG.write('>' + str(allele.id) + '\n' + str(allele.seq.upper()) + '\n')
            fG.close()
            break

    return True


def main(geneFiles):
    #~ parser = argparse.ArgumentParser(
        #~ description="This program prepares a schema for a chewBBACA allele call, creating a short version of each fast with only the 1st allele")
    #~ parser.add_argument('-i', nargs='?', type=str, help='List of genes files (list of fasta files)', required=True)
#~ 
    #~ args = parser.parse_args()

    geneFiles = args.i
    listGenes = []
    gene_fp = open(geneFiles, 'r')
    for gene in gene_fp:
        gene = gene.rstrip('\n')
        listGenes.append(gene)
    gene_fp.close()

    get_Short(listGenes)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
from Bio import SeqIO
import argparse
import plotly
import plotly.graph_objs as go

import math

def calculateN_50(input_file,ratio):

    lengths=[]
    sums=0

    for seq_record in SeqIO.parse(input_file, "fasta"):
        lengths.append(len(seq_record.seq))

    lengths=sorted(lengths, reverse=True)

    contigsMore1000=0
    totalLengthMore1000=0

    for elem in lengths:
        if elem >=10000:
            contigsMore1000+=1
            totalLengthMore1000+=int(elem)
    N_half=sum(lengths)*ratio

    for i in range(0, len(lengths)):
        sums=sums+lengths[i]
        if sums >= N_half:
            return lengths[i],len(lengths),N_half/ratio,contigsMore1000,totalLengthMore1000


def addAnnotations(aux1,aux2,annotationsList,listFiles2Check):
    auxDict={}
    for k in aux1:
        if k in listFiles2Check:
            x=aux1.index(k)
            y=aux2[x]
            aux=dict(
                y=math.log(y,10),
                x=x,
                xref='x',
                yref='y',
                #~ text=k,
                showarrow=True,
                ax=20,
                ay=-40,

            )
            annotationsList.append(aux)
            auxDict[k]=[x,y]
    return auxDict

def main():

    parser = argparse.ArgumentParser(description="do a plot of number of pairs of isolates vs number of allelic differences")
    parser.add_argument('-i', nargs='?', type=str, help='path where genomes are', required=True)
    parser.add_argument('-g', nargs='?', type=str, help='List of genomes to annotate', required=False)


    args = parser.parse_args()
    contigsfile = args.i
    try:
        listGenomes = args.g
    except:
        listGenomes=[]

    results=[]
    dictContigs={}
    dictContigs1000={}
    dictBases={}
    dictBases1000={}
    dictn50={}
    dictn90={}

    try:
        f=open( contigsfile, 'r')
        f.close()
    except IOError:
        listbasename=os.path.basename(os.path.normpath(contigsfile))
        for assembly in os.listdir(contigsfile):
            print("processing: "+assembly)
            try:
                assemblypath=os.path.join(contigsfile,assembly)
                #~ n90,numberContigs,numberBases,contigsMore1000=calculateN_50(assemblypath,0.9)
                n50,numberContigs,numberBases,contigsMore1000,numberBases1000=calculateN_50(assemblypath,0.5)
                results.append([assembly,numberContigs,numberBases,n50])
                dictContigs[assembly]=numberContigs
                dictContigs1000[assembly]=contigsMore1000
                dictBases[assembly]=numberBases
                dictBases1000[assembly]=numberBases1000
                dictn50[assembly]=n50
                #~ dictn90[assembly]=n90
            except:
                print(assembly+ " is not a fasta file")

    listFiles = []

    with open(listGenomes, 'r') as genomes_fp:
        for genome in genomes_fp:
            genome = genome.rstrip('\n')
            genome = genome.rstrip('\r')
            listFiles.append( genome )



    with open("AssemblyStats.tsv", "w") as f:
        f.write("File\tNo.Contigs\tNo.Bases\tN50\n")
        for elem in results:
            f.write('\t'.join(map(str,elem))+"\n")

    #~ sortedKeys90=sorted(dictn90.keys(),key=dictn90.__getitem__,reverse=True)

    if len(listFiles)==0:
        typeScatter=go.Scattergl
    else:
        typeScatter=go.Scatter

        sortedKeys50=sorted(dictn50.keys(),key=dictn50.__getitem__,reverse=True)
        sortedKeysContigs=sorted(dictContigs.keys(),key=dictContigs.__getitem__,reverse=True)
        sortedKeysContigs1000=sorted(dictContigs1000.keys(),key=dictContigs1000.__getitem__,reverse=True)
        sortedKeysBases=sorted(dictBases.keys(),key=dictBases.__getitem__,reverse=True)
        sortedKeysBases1000=sorted(dictBases1000.keys(),key=dictBases1000.__getitem__,reverse=True)
        #~ sortedvalues90=sorted(dictn90.values(),reverse=True)
        sortedvalues50=sorted(dictn50.values(),reverse=True)
        sortedvaluesContigs=sorted(dictContigs.values(),reverse=True)
        sortedvaluesContigs1000=sorted(dictContigs1000.values(),reverse=True)
        sortedvaluesBases=sorted(dictBases.values(),reverse=True)
        sortedvaluesBases1000=sorted(dictBases1000.values(),reverse=True)


    annotations=[]

    annotDict=addAnnotations(sortedKeys50,sortedvalues50,annotations,listFiles)
    annotDict2=addAnnotations(sortedKeysContigs,sortedvaluesContigs,annotations,listFiles)
    #~ addAnnotations(sortedKeysBases,sortedvaluesContigs,annotations,listFiles)
    annotDict3=addAnnotations(sortedKeysBases1000,sortedvaluesBases1000,annotations,listFiles)
    annotDict4=addAnnotations(sortedKeysContigs1000,sortedvaluesContigs1000,annotations,listFiles)

    listTraces=[]

    for k,v in annotDict2.items():
        x1=v[0]
        y1=v[1]
        v=annotDict4[k]
        x2=v[0]
        y2=v[1]

        trace = typeScatter(
            x= [x1,x2],
            y = [y1,y2],
            name= k,
            mode = 'lines',
            hoverinfo= "none",
            showlegend=False,
            line = dict(
                color = 'grey',
                width = 1,
                dash = 'dash')
        )
        listTraces.append(trace)

    for k,v in annotDict.items():
        x1=v[0]
        y1=v[1]
        v=annotDict3[k]
        x2=v[0]
        y2=v[1]

        trace = typeScatter(
            x= [x1,x2],
            y = [y1,y2],
            name= k,
            mode = 'lines',
            hoverinfo= "none",
            showlegend=False,
            line = dict(
                color = 'grey',
                width = 1,
                dash = 'dash')
        )
        listTraces.append(trace)

        #~ for k,v in annotDict.iteritems():
        #~ x1=v[0]
        #~ y1=v[1]
        #~ v=annotDict2[k]
        #~ x2=v[0]
        #~ y2=v[1]
        #~
        #~ trace = go.Scatter(
        #~ x= [x1,x2],
        #~ y = [y1,y2],
        #~ name= k,
        #~ mode = 'lines',
        #~ hoverinfo= "none",
        #~ showlegend=False,
        #~ line = dict(
        #~ color = 'grey',
        #~ width = 1,
        #~ dash = 'dash')
        #~ )
        #~ listTraces.append(trace)


    trace1 = typeScatter(
        y = sorted(dictContigs.values(),reverse=True),
        name= "No. contigs",
        mode = 'markers',
        text = sorted(dictContigs.keys(),key=dictContigs.__getitem__,reverse=True),
        hoverinfo= "y+text",
        marker = dict(
            color = 'pink',
        ),
    )
    listTraces.append(trace1)
    #~ trace2 = go.Scatter(
    #~ y = sorted(dictBases.values(),reverse=True),
    #~ name= "No. bases",
    #~ mode = 'markers',
    #~ text = sorted(dictBases.keys(),key=dictBases.__getitem__,reverse=True),
    #~ )
    #~ trace1 = go.Scatter(
    #~ y = sorted(dictn90.values(),reverse=True),
    #~ name= "N90",
    #~ text = sorted(dictn90.keys(),key=dictn90.__getitem__,reverse=True),
    #~ )
    trace3 = typeScatter(
        y = sorted(dictn50.values(),reverse=True),
        name= "N50",
        mode = 'markers',
        marker = dict(
            color = 'red',
        ),
        text = sorted(dictn50.keys(),key=dictn50.__getitem__,reverse=True),
        hoverinfo= "y+text",
    )
    listTraces.append(trace3)
    trace4 = typeScatter(
        y = sorted(dictContigs1000.values(),reverse=True),
        name= "No. contigs > 10k bp",
        mode = 'markers',
        marker = dict(
            color = 'green',
        ),
        text = sorted(dictContigs1000.keys(),key=dictContigs1000.__getitem__,reverse=True),
        hoverinfo= "y+text",
    )
    listTraces.append(trace4)
    trace5 = typeScatter(
        y = sorted(dictBases1000.values(),reverse=True),
        name= "No. bases >10k bp",
        mode = 'markers',
        hoverinfo= "y+text",
        marker = dict(
            color = 'blue',
        ),
        text = sorted(dictBases1000.keys(),key=dictBases1000.__getitem__,reverse=True),
    )
    listTraces.append(trace5)




    layout = go.Layout(
        hovermode= 'closest',

        xaxis=dict(
            showgrid=False,
            zeroline=False,
            showline=False,
            showticklabels=False
        ),
        yaxis=dict(
            type='log',
            autorange=True
        ),
        annotations=annotations
    )


    fig = go.Figure(data=listTraces, layout=layout)
    plot_url = plotly.offline.plot(fig, filename='AssemblyStatsStack.html')


if __name__ == "__main__":
    main()



from setuptools import setup

import CHEWBBACA

VERSION = CHEWBBACA.__version__

setup(
  name = 'chewBBACA',
  packages = ['CHEWBBACA','CHEWBBACA.allelecall','CHEWBBACA.createschema','CHEWBBACA.utils','CHEWBBACA.SchemaEvaluator'],
  version = VERSION,
  description = 'A complete suite for gene-by-gene schema creation and strain identification',
  author = 'Mickael Silva',
  author_email = 'mickaelsilva@medicina.ulisboa.pt',
  url = 'https://github.com/B-UMMI/chewBBACA', 
  keywords = ['cgMLST', 'bacterial typing'],
  install_requires=['numpy>=1.14.0','scipy>=0.13.3','biopython>=1.70','plotly>=1.12.9','SPARQLWrapper>=1.8.0','pandas>=0.22.0','requests>=2.2.1'],
  python_requires='>=3.4',
  entry_points={
        "console_scripts": [
            "chewBBACA.py = CHEWBBACA.chewBBACA:main",
            "chewie = CHEWBBACA.chewBBACA:main"
        ]
}
)



#!/usr/bin/env python3

import sys
import os
import argparse
from Bio import SeqIO
from Bio.Alphabet import generic_dna

try:
	from allelecall import BBACA
	from createschema import PPanGen
	from SchemaEvaluator import ValidateSchema
	from utils import TestGenomeQuality,profile_joiner,init_schema_4_bbaca,uniprot_find,Extract_cgAlleles,RemoveGenes
except ImportError:
	from CHEWBBACA.allelecall import BBACA
	from CHEWBBACA.createschema import PPanGen
	from CHEWBBACA.SchemaEvaluator import ValidateSchema
	from CHEWBBACA.utils import TestGenomeQuality,profile_joiner,init_schema_4_bbaca,uniprot_find,Extract_cgAlleles,RemoveGenes

#~ from allelecall import CommonFastaFunctions,callAlleles_protein3,BBACA


def check_if_list_or_folder(folder_or_list):
    list_files = []
    # check if given a list of genomes paths or a folder to create schema
    try:
        f = open(folder_or_list, 'r')
        f.close()
        list_files = folder_or_list
    except IOError:
		
        for gene in os.listdir(folder_or_list):
			
            try:
                genepath = os.path.join(folder_or_list, gene)
                
                if os.path.isdir(genepath):
                    continue
                
                for allele in SeqIO.parse(genepath, "fasta", generic_dna):
                    break
                list_files.append(os.path.abspath(genepath))
            except Exception as e:
                print(e)
                pass

    return list_files


def create_schema():

    def msg(name=None):                                                            
        return ''' chewBBACA.py CreateSchema [CreateSchema ...] [-h] -i [I] -o [O] --cpu [CPU] [-b [B]] [--bsr [BSR]] [-v] [-l [L]]'''

    parser = argparse.ArgumentParser(description="This program creates a schema when provided the genomes",usage=msg())
    parser.add_argument('CreateSchema', nargs='+', help='create a schema')
    parser.add_argument('-i', nargs='?', type=str, help='List of genome files (list of fasta files)', required=True)
    parser.add_argument('-o', nargs='?', type=str, help="Name of the output folder", required=True)
    parser.add_argument('--cpu', nargs='?', type=int, help="Number of cpus, if over the maximum uses maximum -2",
                        required=True)
    parser.add_argument('-b', nargs='?', type=str, help="BLAST full path", required=False, default='blastp')
    parser.add_argument("--CDS", help="use a fasta file, one gene per entry, to call alleles", required=False, action="store_true", default=False)
    parser.add_argument('--bsr', nargs='?', type=float, help="minimum BSR similarity", required=False, default=0.6)
    #~ parser.add_argument('-t', nargs='?', type=str, help="taxon", required=False, default=False)
    parser.add_argument('--ptf', nargs='?', type=str, help="provide your own prodigal training file (ptf) path", required=False, default=False)
    parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
                        default=False)
    parser.add_argument('-l', nargs='?', type=int, help="minimum bp locus lenght", required=False, default=201)

    args = parser.parse_args()

    genomeFiles = args.i
    cpuToUse = args.cpu
    outputFile = args.o
    BlastpPath = args.b
    bsr = args.bsr
    #~ chosenTaxon = args.t
    chosenTrainingFile = args.ptf
    verbose = args.verbose
    min_length = args.l
    inputCDS = args.CDS

    if inputCDS==True:
        genomeFiles=[os.path.abspath(genomeFiles)]
    else:
        genomeFiles = check_if_list_or_folder(genomeFiles)

    if isinstance(genomeFiles, list):
        with open("listGenomes2Call.txt", "w") as f:
            for genome in genomeFiles:
                f.write(genome + "\n")
        genomeFiles = "listGenomes2Call.txt"

    
    PPanGen.main(genomeFiles,cpuToUse,outputFile,bsr,BlastpPath,min_length,verbose,chosenTrainingFile,inputCDS)

    
    
    try:
        os.remove("listGenomes2Call.txt")
    except:
        pass


def allele_call():

    def msg(name=None):                                                            
        return ''' chewBBACA.py AlleleCall [AlleleCall ...][-h] -i [I] -g [G] -o [O] --cpu [CPU] [-v] [-b [B]][--bsr [BSR]] [--ptf [PTF]] [--fc] [--fr] [--json]
            '''

    parser = argparse.ArgumentParser(description="This program call alleles for a set of genomes when provided a schema",usage=msg())
    parser.add_argument('AlleleCall', nargs='+', help='do allele call')
    parser.add_argument('-i', nargs='?', type=str, help='List of genome files (list of fasta files)', required=True)
    parser.add_argument('-g', nargs='?', type=str, help='List of genes (fasta)', required=True)
    parser.add_argument('-o', nargs='?', type=str, help="Name of the output files", required=True)
    parser.add_argument('--cpu', nargs='?', type=int, help="Number of cpus, if over the maximum uses maximum -2",
                        required=True)
    parser.add_argument("--contained", help=argparse.SUPPRESS, required=False, action="store_true", default=False)
    parser.add_argument("--CDS", help=argparse.SUPPRESS, required=False, action="store_true", default=False)
    parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
                        default=False)
    parser.add_argument('-b', nargs='?', type=str, help="BLAST full path", required=False, default='blastp')
    parser.add_argument('--bsr', nargs='?', type=float, help="minimum BSR score", required=False, default=0.6)
    parser.add_argument('--st', nargs='?', type=float, help="size threshold, default at 0.2 means alleles with size variation of +-20 percent will be tagged as ASM/ALM", required=False, default=0.2)
    #parser.add_argument('-t', nargs='?', type=str, help="taxon", required=False, default=False)
    parser.add_argument('--ptf', nargs='?', type=str, help="provide the prodigal training file (ptf) path", required=False, default=False)
    parser.add_argument("--fc", help="force continue", required=False, action="store_true", default=False)
    parser.add_argument("--fr", help="force reset", required=False, action="store_true", default=False)
    parser.add_argument("--json", help="report in json file", required=False, action="store_true", default=False)

    args = parser.parse_args()

    genomeFiles = args.i
    genes = args.g
    cpuToUse = args.cpu
    BSRTresh = args.bsr
    sizeTresh = args.st
    verbose = args.verbose
    BlastpPath = args.b
    gOutFile = args.o
    chosenTaxon = False
    chosenTrainingFile = args.ptf
    forceContinue = args.fc
    forceReset = args.fr
    contained = args.contained
    inputCDS = args.CDS
    jsonReport = args.json

    genes2call = check_if_list_or_folder(genes)

    if isinstance(genes2call, list):
        with open("listGenes2Call.txt", "w") as f:
            for genome in genes2call:
                f.write(genome + "\n")
        genes2call = "listGenes2Call.txt"

    
    #try to open as a fasta
    fasta = SeqIO.parse(genomeFiles, "fasta", generic_dna)
    try:
        isFasta=(any(fasta))
    except:
        isFasta=False
    
    #if is a fasta pass as a list of genomes with a single genome, if not check if is a folder or a txt with a list of paths
    if isFasta==True:
        genomes2call=[os.path.abspath(genomeFiles)]
    else:
        genomes2call = check_if_list_or_folder(genomeFiles)
    
    if isinstance(genomes2call, list):
        with open("listGenomes2Call.txt", "w") as f:
            for genome in genomes2call:
                f.write(genome + "\n")
        genomes2call = "listGenomes2Call.txt"

    
    BBACA.main(genomes2call,genes2call,cpuToUse,gOutFile,BSRTresh,BlastpPath,forceContinue,jsonReport,verbose,forceReset,contained,chosenTaxon,chosenTrainingFile,inputCDS,sizeTresh)


    try:
        os.remove("listGenes2Call.txt")
    except:
        pass
    try:
        os.remove("listGenomes2Call.txt")
    except:
        pass

def evaluate_schema():

    def msg(name=None):                                                            
        return ''' chewBBACA.py SchemaEvaluator [SchemaEvaluator ...] [-h] -i [I] [-p] [--log] -l [L] -ta [TA] [-t [T]] [--title [TITLE]] --cpu [CPU] [-s [S]] [--light]
            '''

    parser = argparse.ArgumentParser(
        description="This program analyses a set of gene files, analyzing the alleles CDS and the length of the alleles per gene",usage=msg())
    parser.add_argument('SchemaEvaluator', nargs='+', help='evaluation of a schema')
    parser.add_argument('-i', nargs='?', type=str, help='list genes, directory or .txt file with the full path',
                        required=True)
    parser.add_argument('-p', dest='conserved', action='store_true', help='One bad allele still makes gene conserved',
                        required=False,
                        default=False)
    parser.add_argument('--log', dest='logScale', action='store_true', default=False)
    parser.add_argument('-l', nargs='?', type=str, help='name/location main html file', required=True)
    parser.add_argument('-ta', nargs='?', type=int, help='ncbi translation table', required=False, default=11)
    parser.add_argument('-t', nargs='?', type=float, help='Threshold', required=False, default=0.05)
    parser.add_argument('--title', nargs='?', type=str, help='title on the html', required=False,
                        default="My Analyzed wg/cg MLST Schema - Rate My Schema")
    parser.add_argument('--cpu', nargs='?', type=int, help='number of cpu to use', required=True)
    parser.add_argument('-s', nargs='?', type=int,
                        help='number of boxplots per page (more than 500 can make the page very slow)', required=False,
                        default=500)
    parser.add_argument('--light', help="skip clustal and mafft run", required=False, action="store_true", default=False)
    
    args = parser.parse_args()
    genes = args.i
    transTable = args.ta
    logScale = args.logScale
    outputpath = args.l
    cpuToUse = args.cpu
    threshold = args.t
    OneBadGeneNotConserved = args.conserved
    splited = args.s
    light = args.light
    title = str(args.title)

    ValidateSchema.main(genes,cpuToUse,outputpath,transTable,threshold,splited,title,logScale,OneBadGeneNotConserved,light)



def test_schema():

    def msg(name=None):                                                            
        return ''' chewBBACA.py TestGenomeQuality [TestGenomeQuality ...] [-h] -i [I] -n [N] -t [T] -s [S] [-o [O]] [-v]
                    '''

    parser = argparse.ArgumentParser(
        description="This program analyze an allele call raw output matrix, returning info on which genomes are responsible for cgMLST loci loss",usage=msg())
    parser.add_argument('TestGenomeQuality', nargs='+', help='test the quality of the genomes on the allele call')
    parser.add_argument('-i', nargs='?', type=str, help='raw allele call matrix file', required=True)
    parser.add_argument('-n', nargs='?', type=int, help='maximum number of iterations', required=True)
    parser.add_argument('-t', nargs='?', type=int, help='maximum threshold of bad calls above 95 percent',
                        required=True)
    parser.add_argument('-s', nargs='?', type=int, help='step between each threshold analysis', required=True)
    parser.add_argument('-o', nargs='?', type=str, help="Folder for the analysis files", required=False, default=".")
    parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
                        default=False)

    args = parser.parse_args()

    pathOutputfile = args.i
    iterationNumber = args.n
    thresholdBadCalls = args.t
    step = args.s
    out_folder = args.o
    verbose = args.verbose

    TestGenomeQuality.main(pathOutputfile,iterationNumber,thresholdBadCalls,step,out_folder,verbose)



def extract_cgmlst():

    def msg(name=None):                                                            
        return ''' chewBBACA.py ExtractCgMLST [ExtractCgMLST ...] [-h] -i [I] -o [O] [-r [R]] [-g [G]]
                    '''

    parser = argparse.ArgumentParser(description="This program cleans an output file for phyloviz",usage=msg())
    parser.add_argument('ExtractCgMLST', nargs='+', help='clean chewBBACA output')
    parser.add_argument('-i', nargs='?', type=str, help='input file to clean', required=True)
    parser.add_argument('-o', nargs='?', type=str, help='output folder', required=True)
    parser.add_argument('-r', nargs='?', type=str, help='listgenes to remove', required=False, default=False)
    parser.add_argument('-g', nargs='?', type=str, help='listgenomes to remove', required=False, default=False)
    parser.add_argument('-p', nargs='?', type=float, help='maximum presence (e.g 0.95)', required=False, default=1)

    args = parser.parse_args()

    pathOutputfile = args.i
    newfile = args.o
    genes2remove = args.r
    genomes2remove = args.g
    cgMLSTpercent=args.p

    Extract_cgAlleles.main(pathOutputfile,newfile,cgMLSTpercent,genes2remove,genomes2remove)


def remove_genes():

    def msg(name=None):                                                            
        return ''' chewBBACA.py RemoveGenes [RemoveGenes ...][-h] -i [I] -g [G] -o [O] [--inverse]
                    '''

    parser = argparse.ArgumentParser(description="This program removes gens from a tab separated allele profile file",usage=msg())
    parser.add_argument('RemoveGenes', nargs='+', help='remove loci from a chewBBACA profile')
    parser.add_argument('-i', nargs='?', type=str, help='main matrix file from which to remove', required=True)
    parser.add_argument('-g', nargs='?', type=str, help='list of genes to remove', required=True)
    parser.add_argument('-o', nargs='?', type=str, help='output file name', required=True)
    parser.add_argument("--inverse", help="list to remove is actually the one to keep", dest='inverse',
                        action="store_true", default=False)

    args = parser.parse_args()

    args = parser.parse_args()
    mainListFile = args.i
    toRemoveListFile = args.g
    outputfileName = args.o
    inverse = args.inverse
    
    
    RemoveGenes.main(mainListFile,toRemoveListFile,outputfileName,inverse)


def join_profiles():

    def msg(name=None):
        return ''' chewBBACA.py JoinProfiles [RemoveGenes ...][-h] -p1 -p2 -o [O] 
                    '''

    parser = argparse.ArgumentParser(description="This program joins two profiles, returning a single profile file with the common loci",usage=msg())
    parser.add_argument('JoinProfiles', nargs='+', help='join profiles')
    parser.add_argument('-p1', nargs='?', type=str, help='profile 1', required=True)
    parser.add_argument('-p2', nargs='?', type=str, help='profile 2', required=True)
    parser.add_argument('-o', nargs='?', type=str, help='outut file name', required=True)

    args = parser.parse_args()
    profile1 = args.p1
    profile2 = args.p2
    outputFile = args.o

    profile_joiner.main(profile1,profile2,outputFile)


def prep_schema():

    def msg(name=None):                                                            
        return ''' chewBBACA.py PrepExternalSchema [PrepExternalSchema ...][-h] -i [I] --cpu [CPU] [-v]
                    '''

    parser = argparse.ArgumentParser(
        description="This program prepares a schema for a chewBBACA allele call, creating a short version of each fast with only the 1st allele")
    parser.add_argument('PrepExternalSchema', nargs='+', help='prepare a schema for chewbbaca')
    parser.add_argument('-i', nargs='?', type=str, help='path to folder containg the schema fasta files ( alternative a list of fasta files)', required=True)
    parser.add_argument('--cpu', nargs='?', type=int, help='number of cpu', required=False, default=1)
    parser.add_argument("-v", "--verbose", help="increase output verbosity", dest='verbose', action="store_true",
                        default=False)

    args = parser.parse_args()

    geneFiles = args.i
    cpu2use = args.cpu
    verbose = args.verbose
    
    init_schema_4_bbaca.main(geneFiles,cpu2use)


def find_uniprot():
	
    def msg(name=None):                                                            
        return ''' chewBBACA.py UniprotFinder [UniprotFinder ...][-h] -i [I] -t [T] --cpu [CPU]
                    '''
	
    parser = argparse.ArgumentParser(
        description="This program gets information of each locus created on the schema creation, based on the uniprot database")
    parser.add_argument('UniprotFinder', nargs='+', help='get info about a schema created with chewBBACA')
    parser.add_argument('-i', nargs='?', type=str, help='path to folder containg the schema fasta files ( alternative a list of fasta files)', required=True)
    parser.add_argument('-t', nargs='?', type=str, help='path to proteinID_Genome.tsv file generated', required=True)
    parser.add_argument('--cpu', nargs='?', type=int, help='number of cpu', required=False, default=1)

    args = parser.parse_args()

    geneFiles = args.i
    tsvFile = args.t
    cpu2use = args.cpu
    
    uniprot_find.main(geneFiles,tsvFile,cpu2use)


def main():
    functions_list = ['CreateSchema', 'AlleleCall', 'SchemaEvaluator', 'TestGenomeQuality', 'ExtractCgMLST','RemoveGenes','PrepExternalSchema','JoinProfiles','UniprotFinder']
    desc_list = ['Create a gene by gene schema based on genomes', 'Perform allele call for target genomes', 'Tool that builds an html output to better navigate/visualize your schema', 'Analyze your allele call output to refine schemas', 'Select a subset of loci without missing data (to be used as PHYLOViZ input)','Remove a provided list of loci from your allele call output','prepare an external schema to be used by chewBBACA','join two profiles in a single profile file','get info about a schema created with chewBBACA']

    version="2.0.13"
    createdBy="Mickael Silva"
    rep="https://github.com/B-UMMI/chewBBACA"
    contact="mickaelsilva@medicina.ulisboa.pt"

    if len(sys.argv)>1 and "version" in sys.argv[1]:
        print (version)
        return
    
    print ("chewBBACA version "+version+" by "+ createdBy+ " at "+ rep+ "\nemail contact: "+ contact)

    try:
        print ("\n")
        if sys.argv[1] == functions_list[0]:
            create_schema()
        elif sys.argv[1] == functions_list[1]:
            allele_call()
        elif sys.argv[1] == functions_list[2]:
            evaluate_schema()
        elif sys.argv[1] == functions_list[3]:
            test_schema()
        elif sys.argv[1] == functions_list[4]:
            extract_cgmlst()
        elif sys.argv[1] == functions_list[5]:
            remove_genes()
        elif sys.argv[1] == functions_list[6]:
            prep_schema()
        elif sys.argv[1] == functions_list[7]:
            join_profiles()
        elif sys.argv[1] == functions_list[8]:
            find_uniprot()
        else:
            print('\n\tUSAGE : chewBBACA.py [module] -h \n')
            print('Select one of the following functions :\n')
            i=0
            while i<len(functions_list):
                print (functions_list[i] +" : "+desc_list[i])
                i+=1
    except Exception as e:
        print (e)
        print('\n\tUSAGE : chewBBACA.py [module] -h \n')
        print('Select one of the following functions :\n')
        i=0
        while i<len(functions_list):
            print (functions_list[i] +" : "+desc_list[i])
            i+=1
if __name__ == "__main__":
    main()
